# Licensed to the .NET Foundation under one or more agreements.
# The .NET Foundation licenses this file to you under the MIT license.

from __future__ import annotations

import json
import os
from typing import Optional

from . import ConfigurationProvider
from .abstractions import ConfigurationPath, IConfigurationBuilder, IConfigurationProvider, IConfigurationSource

class JsonConfigurationParser:
    _data: dict[str, Optional[str]]
    _paths: list[str]

    def __init__(self):
        self._data = dict()
        self._paths = list()

    @staticmethod
    def parse_file(path: str) -> dict[str, Optional[str]]:
        if os.path.isfile(path):
            with open(path, encoding="utf-8-sig") as file:
                try:
                    return JsonConfigurationParser().parse(json.load(file))
                except:
                    return dict()

        return dict()

    @staticmethod
    def parse_string(json_string: str) -> dict[str, Optional[str]]:
        return JsonConfigurationParser().parse(json.loads(json_string))

    def parse(self, input: dict[str, any]) -> dict[str, Optional[str]]:
        self.visit_object_element(input)
        return self._data
    
    def visit_object_element(self, element: dict[str, any]) -> None:
        is_empty: bool = True

        for key, value in element.items():
            is_empty = False
            self.enter_context(key)
            self.visit_value(value)
            self.exit_context()
    
        self.set_null_if_element_is_empty(is_empty)

    def visit_array_element(self, element: any) -> None:
        index: int = 0

        for array_element in element:
            self.enter_context(str(index))
            self.visit_value(array_element)
            self.exit_context()
            index += 1

        self.set_null_if_element_is_empty(index == 0)

    def set_null_if_element_is_empty(self, is_empty: bool) -> None:
        if (is_empty and len(self._paths) > 0):
            self._data[self._paths[-1]] = None

    def visit_value(self, value: any) -> None:
        if isinstance(value, dict):
            self.visit_object_element(value)
        elif isinstance(value, list):
            self.visit_array_element(value)
        else:
            key: str = self._paths[-1]
            if next((other_key for other_key in self._data.keys() if other_key.casefold() == key.casefold()), None) is not None:
                raise ValueError("Key is duplicated.")
            self._data[key] = str(value)
    
    def enter_context(self, context: str) -> None:
        self._paths.append(self._paths[-1] + ConfigurationPath.KEY_DELIMITER + context if len(self._paths) > 0 else context)

    def exit_context(self) -> None:
        self._paths.pop()

class JsonFileConfigurationSource(IConfigurationSource):
    def __init__(self, path: str, optional: bool = True):
        self.path = path
        self.optional = optional

    def build(self, builder: IConfigurationBuilder) -> IConfigurationProvider:
        return JsonFileConfigurationProvider(self.path, self.optional)

class JsonFileConfigurationProvider(ConfigurationProvider):
    def __init__(self, path: str, optional: bool):
        super().__init__()
        self.path = path
        self.optional = optional

    def load(self) -> None:
        if not os.path.exists(self.path):
            if not self.optional:
                raise FileNotFoundError(f"The configuration file '{self.path}' was not found and is not optional.")
            self.data = {}
        else:
            self.data = JsonConfigurationParser.parse_file(self.path)

class JsonStringConfigurationProvider(ConfigurationProvider):
    def __init__(self, json_string: str):
        super().__init__()
        self.json_string = json_string

    def load(self) -> None:
        self.data = JsonConfigurationParser.parse_string(self.json_string)

class JsonStringConfigurationSource(IConfigurationSource):
    def __init__(self, json_string: str):
        self.json_string = json_string

    def build(self, builder: IConfigurationBuilder) -> IConfigurationProvider:
        return JsonStringConfigurationProvider(self.json_string)

def _add_json_file(self, file_path: str, optional: bool = True) -> IConfigurationBuilder:
    self.add(JsonFileConfigurationSource(file_path, optional))
    return self

def _add_json_string(self, json_string: str) -> IConfigurationBuilder:
    self.add(JsonStringConfigurationSource(json_string))
    return self

IConfigurationBuilder.add_json_file = _add_json_file
IConfigurationBuilder.add_json_string = _add_json_string 


# Licensed to the .NET Foundation under one or more agreements.
# The .NET Foundation licenses this file to you under the MIT license.

from __future__ import annotations

from typing import Optional

from abc import ABC, abstractmethod
from collections.abc import Iterable

class ConfigurationPath:
    """Utility methods and constants for manipulating Configuration paths"""
    KEY_DELIMITER: str = ":"

    @staticmethod
    def combine(*path_segments: str) -> str:
        """Combines path segments into one path.
        
        :param list[str] path_segments: The path segments to combine.
        :return: The combined path"""
        return ConfigurationPath.KEY_DELIMITER.join(path_segments)

    @staticmethod
    def get_section_key(path: Optional[str]) -> Optional[str]:
        """Extracts the last path segment from the path.
        
        :param str path: The path.
        :return: The last path segment of the path."""
        if not path:
            return path
        
        last_delimiter_index: int = path.rfind(ConfigurationPath.KEY_DELIMITER)
        return path if last_delimiter_index < 0 else path[last_delimiter_index + 1:]
    
    @staticmethod
    def get_parent_path(path: Optional[str]) -> Optional[str]:
        """Extracts the path corresponding to the parent node for a given path.
        
        :param str path: The path.
        :rtype: Optional[str]
        :return: The original path minus the last individual segment found in it. Null if the original path corresponds to a top level node."""
        if not path:
            return None
        
class IComparer(ABC):
    def compare(x: any, y: any) -> int:
        """Compares two objects and returns a value indicating whether one is less than, equal to, or greater than the other."""
        pass

class IConfiguration(ABC):
    @abstractmethod
    def __getitem__(self, key: str) -> str:
        pass

    @abstractmethod
    def __setitem__(self, key: str, value: str) -> None:
        pass

    @abstractmethod
    def get_section(self, key: str) -> IConfigurationSection:
        """Gets a configuration sub-section with the specified key.
        
        :param str key: The key of the configuration section."""
        pass

    @abstractmethod
    def get_children(self) -> Iterable[IConfigurationSection]:
        """Gets the immediate descendant configuration sub-sections.
        
        :return: The configuration sub-sections."""
        pass

    def get_connection_string(self, name: str) -> Optional[str]:
        """Shorthand for get_section("ConnectionStrings")[name]."""
        return self.get_section("ConnectionStrings")[name]

class IConfigurationBuilder(ABC):
    @abstractmethod
    def properties(self) -> dict[str, object]:
        pass

    @property
    @abstractmethod
    def sources(self) -> list[IConfigurationSource]:
        pass

    @abstractmethod
    def add(self, source: IConfigurationSource) -> IConfigurationBuilder:
        """Adds a new configuration source.
        
        :param IConfigurationSource source: The configuration source to add."""
        pass

    @abstractmethod
    def build(self) -> IConfigurationRoot:
        """Builds an IConfiguration with keys and values from the set of sources registered in"""
        pass
    
class IConfigurationProvider(ABC):
    """Provides configuration key/values for an application."""
    @abstractmethod
    def try_get(self, key: str) -> tuple[bool, Optional[str]]:
        """Tries to get a configuration value for the specified key.
        
        :param str key: The key.
        :return: True if a value for the specified key was found, otherwise False.
        """
        pass

    @abstractmethod
    def set(self, key: str, value: str) -> None:
        """Sets a configuration value for the specified key.
        
        :param str key: The key.
        :param str value: The value."""
        pass

    @abstractmethod
    def load(self) -> None:
        """Loads configuration values from the source represented by this IConfigurationProvider."""
        pass

    @abstractmethod
    def get_child_keys(self, earlier_keys: Iterable[str], parent_path: str) -> Iterable[str]:
        """Returns the immediate descendant configuration keys for a given parent path based on this IConfigurationProviders data and the set of keys returned by all the preceding IConfigurationProviders.
        
        :param Iterable[str] earlier_keys: The child keys returned by the preceding providers for the same parent path.
        :param str parent_path: The parent path.
        :return: The child keys.
        """
        pass

class IConfigurationRoot(IConfiguration):
    """Represents the root of an IConfiguration hierarchy."""
    @abstractmethod
    def reload(self):
        """Force the configuration values to be reloaded from the underlying IConfigurationProviders."""
        pass
    
    @property
    @abstractmethod
    def providers(self) -> Iterable[IConfigurationProvider]:
        """The IConfigurationProviders for this configuration."""
        pass

class IConfigurationSection(IConfiguration):
    """Represents a section of application configuration values."""
    @property
    @abstractmethod
    def key(self) -> str:
        """Gets the key this section occupies in its parent."""
        pass

    @property
    @abstractmethod
    def path(self) -> str:
        """Gets the full path to this section within the IConfiguration."""
        pass

    @property
    @abstractmethod
    def value(self) -> str:
        pass

    @value.setter
    @abstractmethod
    def value(self, value: str) -> None:
        pass

class IConfigurationSource(ABC):
    """Represents a source of configuration key/values for an application."""
    @abstractmethod
    def build(self, builder: IConfigurationBuilder) -> IConfigurationProvider:
        """Builds the IConfigurationProvider for this source.
        
        :param IConfigurationBuilder builder: The IConfigurationBuilder.
        :return: An IConfigurationProvider.
        """
        pass


# Licensed to the .NET Foundation under one or more agreements.
# The .NET Foundation licenses this file to you under the MIT license.

from __future__ import annotations

from collections.abc import Iterable
from functools import cmp_to_key
from typing import Callable, Optional

from .abstractions import ConfigurationPath, IComparer, IConfiguration, IConfigurationBuilder, IConfigurationProvider, IConfigurationRoot, IConfigurationSection, IConfigurationSource

class ConfigurationKeyComparer(IComparer):
    """IComparer implementation used to order configuration keys."""
    KEY_DELIMITER: str = ":"
    instance: ConfigurationKeyComparer
    comparison: Callable[[Optional[str], Optional[str]], int]

    def compare(self, x: Optional[str], y: Optional[str]) -> int:
        def skip_ahead_on_delimiter(a: str):
            while a and a[0] == ConfigurationKeyComparer.KEY_DELIMITER:
                a = a[1:]
            return a
        
        def compare(a: str, b:str):
            def try_parse(str: str) -> tuple[bool, Optional[int]]:
                value: Optional[int] = None
                try:
                    value = int(str)
                    return True, value
                except:
                    return False, None

            a_is_int, value1 = try_parse(a)
            b_is_int, value2 = try_parse(b)

            result: int

            if not a_is_int and not b_is_int:
                transformed_a = a.casefold()
                transformed_b = b.casefold()

                if transformed_a == transformed_b:
                    result = 0
                else:
                    result = -1 if transformed_a < transformed_b else 1
            elif a_is_int and b_is_int:
                result = value1 - value2
            else:
                result = -1 if a_is_int else 1

            return result
        
        x_span: Optional[str] = x
        y_span: Optional[str] = y

        x_span = skip_ahead_on_delimiter(x_span)
        y_span = skip_ahead_on_delimiter(y_span)

        while x_span and y_span:
            x_delimiter_index: int = -1
            if ConfigurationKeyComparer.KEY_DELIMITER in x_span:
                x_delimiter_index = x_span.index(ConfigurationKeyComparer.KEY_DELIMITER)
            y_delimiter_index: int = -1
            if ConfigurationKeyComparer.KEY_DELIMITER in y_span:
                y_delimiter_index = y_span.index(ConfigurationKeyComparer.KEY_DELIMITER)

            compare_result: int = compare(x_span if x_delimiter_index == -1 else x_span[:x_delimiter_index], y_span if y_delimiter_index == -1 else y_span[:y_delimiter_index])

            if compare_result != 0:
                return compare_result
            
            x_span = None if x_delimiter_index == -1 else skip_ahead_on_delimiter(x_span[x_delimiter_index + 1:])
            y_span = None if y_delimiter_index == -1 else skip_ahead_on_delimiter(y_span[y_delimiter_index + 1:])

        return (0 if not y_span else -1) if not x_span else 1
    
ConfigurationKeyComparer.instance = ConfigurationKeyComparer()
ConfigurationKeyComparer.comparison = ConfigurationKeyComparer.instance.compare

class ConfigurationBuilder(IConfigurationBuilder):
    """Used to build key/value based configuration settings for use in an application."""
    _properties: dict[str, object]
    _sources: list[IConfigurationSource]

    def __init__(self):
        self._properties = dict()
        self._sources = list()

    @property
    def sources(self) -> list[IConfigurationSource]:
        """Returns the sources used to obtain configuration values."""
        return self._sources

    @property
    def properties(self) -> dict[str, object]:
        """Gets a key/value collection that can be used to share data between the ConfigurationBuilder and the registered ConfigurationProviders."""
        return self._properties

    def add(self, source: IConfigurationSource) -> IConfigurationBuilder:
        """Adds a new configuration source.
        
        :param IConfigurationSource source: The configuration source to add.
        :return: The same ConfigurationBuilder.
        """
        if source is None:
            raise ValueError("source cannot be None.")
        
        self._sources.append(source)

        return self
    
    def build(self) -> IConfigurationRoot:
        providers: list[IConfigurationProvider] = list()
        for source in self._sources:
            provider = source.build(self)
            providers.append(provider)

        return ConfigurationRoot(providers)

class ConfigurationProvider(IConfigurationProvider):
    """Base helper class for implementing an IConfigurationProvider"""
    data: dict[str, Optional[str]]

    def __init__(self):
        self.data = dict()

    def try_get(self, key: str) -> tuple[bool, Optional[str]]:
        """Attempts to find a value with the given key, returns True if one is found, False otherwise."""
        transformed_key: str = key.casefold()
        matching_key: Optional[str] = next((key for key in self.data.keys() if key.casefold() == transformed_key), None)
        if matching_key:
            return True, self.data[matching_key]
        else:
            return False, None
        
    def set(self, key: str, value: Optional[str]) -> None:
        """Sets a value for a given key.
        
        :param str key: The configuration key to set.
        :param Optional[str] value: The value to set."""
        self.data[key] = value

    def load() -> None:
        pass

    def get_child_keys(self, earlier_keys: Iterable[str], parent_path: Optional[str]) -> Iterable[str]:
        results: list[str] = list()

        if parent_path is None:
            key: str
            for key in self.data.keys():
                results.append(ConfigurationProvider.segment(key, 0))
        else:
            key: str
            for key in self.data.keys():
                if (len(key) > len(parent_path) and 
                    key.casefold().startswith(parent_path.casefold()) and 
                    key[len(parent_path)] == ConfigurationPath.KEY_DELIMITER):
                    results.append(ConfigurationProvider.segment(key, len(parent_path) + 1))

        results.extend(earlier_keys)
        results = sorted(results, key=cmp_to_key(ConfigurationKeyComparer.comparison))

        return results
    
    @staticmethod
    def segment(key: str, prefix_length: int):
        try:
            index_of: int = key.index(ConfigurationPath.KEY_DELIMITER, prefix_length)
            return key[prefix_length:index_of]
        except ValueError:
            return key[prefix_length]
    
class ConfigurationRoot(IConfigurationRoot):
    """The root node for a configuration."""
    _providers: list[IConfigurationProvider]
    
    def __init__(self, providers: list[IConfigurationProvider]):
        """Initializes a Configuration root with a list of providers."""
        if providers is None:
            raise ValueError("providers cannot be None.")

        self._providers = providers
        provider: IConfigurationProvider
        for provider in providers:
            provider.load()

    @property
    def providers(self) -> Iterable[IConfigurationProvider]:
        """The IConfigurationProviders for this configuration."""
        return self._providers
    
    def __getitem__(self, key: str) -> Optional[str]:
        """Gets the value corresponding to a configuration key."""
        return self.get_configuration(self._providers, key)
    
    def __setitem__(self, key: str, value: Optional[str]) -> None:
        """Sets the value corresponding to a configuration key."""
        self.set_configuration(self._providers, key, value)

    def get_children(self) -> Iterable[IConfigurationSection]:
        """Gets the immediate children sub-sections."""
        return self.get_children_implementation(None)
    
    def get_section(self, key: str) -> IConfigurationSection:
        """Gets a configuration sub-section with the specified key.
        
        :param str key: The key of the configuration section.
        :returns: The IConfigurationSection.
        :rtype: IConfigurationSection"""
        return ConfigurationSection(self, key)
    
    def reload(self) -> None:
        """Force the configuration values to be reloaded from the underlying sources."""
        provider: IConfigurationProvider
        for provider in self._providers:
            provider.load()

    @staticmethod
    def get_configuration(providers: list[IConfigurationProvider], key: str) -> Optional[str]:
        i: int
        for i in range(len(providers) - 1, -1, -1):
            provider: IConfigurationProvider = providers[i]

            condition: bool
            value: Optional[str]
            condition, value = provider.try_get(key)
            if (condition):
                return value

        return None
    
    @staticmethod
    def set_configuration(providers: list[IConfigurationProvider], key: str, value: Optional[str]) -> None:
        if len(providers) == 0:
            raise ValueError("No configuration sources were configured.")
        
        provider: IConfigurationProvider
        for provider in providers:
            provider.set(key, value)

    def get_children_implementation(root: IConfigurationRoot, path: Optional[str]) -> Iterable[IConfigurationSection]:
        """Gets the immediate children sub-sections of configuration root based on key."""
        providers: Iterable[IConfigurationProvider] = root.providers

        child_keys: list[str] = list()
        provider: IConfigurationProvider
        for provider in providers:
            child_keys.extend(provider.get_child_keys(child_keys, path))
        unique_child_keys = list()
        child_key: str
        for child_key in child_keys:
            transformed_child_key = child_key.casefold()
            if not any(unique_key.casefold() == transformed_child_key for unique_key in unique_child_keys):
                unique_child_keys.append(child_key)

        children: list[IConfigurationSection] = list()
        key: str
        for key in unique_child_keys:
            children.append(root.get_section(key if not path else path + ConfigurationPath.KEY_DELIMITER + key))

        return children
    
class ConfigurationSection(IConfigurationSection):
    """Represents a section of application configuration values."""
    _root: IConfigurationRoot
    _path: str
    _key: Optional[str]

    def __init__(self, root: IConfigurationRoot, path: str):
        self._root = root
        self._path = path
        self._key = None

    @property
    def path(self) -> str:
        """Gets the full path to this section from the IConfigurationRoot."""
        return self._path
    
    @property
    def key(self) -> str:
        """Gets the key this section occupies in its parent."""
        # Key is calculated lazily as last portion of Path
        if self._key is None:
            self._key = ConfigurationPath.get_section_key(self._path)

        return self._key
    
    @property
    def value(self) -> str:
        """Gets the section value."""
        return self._root[self.path]
    
    @value.setter
    def value(self, value: str) -> None:
        """Sets the section value."""
        self._root[self.path] = value

    def __getitem__(self, key: str) -> Optional[str]:
        return self._root[self.path + ConfigurationPath.KEY_DELIMITER + key]
    
    def __setitem__(self, key: str, value: Optional[str]) -> None:
        self._root[self.path + ConfigurationPath.KEY_DELIMITER + key] = value

    def get_section(self, key: str) -> IConfigurationSection:
        """Gets a configuration sub-section with the specified key.
        
        :param str key: The key of the configuration section.
        :returns: The IConfigurationSection.
        :rtype: IConfigurationSection"""
        return self._root.get_section(self.path + ConfigurationPath.KEY_DELIMITER + key)
    
    def get_children(self) -> Iterable[IConfigurationSection]:
        """Gets the immediate descendant configuration sub-sections."""
        return self._root.get_children_implementation(self.path)


import os
from uuid import uuid4

import pytest

from universal_common_configuration import ConfigurationBuilder, IConfiguration, IConfigurationSection
import universal_common_configuration.environment_variables
import universal_common_configuration.json
import universal_common_configuration.user_secrets

class TestConfiguration:
    def test_create_configuration_environment_variables(self):
        random_string: str = str(uuid4())
        os.environ["TEST"] = random_string
        configuration: IConfiguration = ConfigurationBuilder().add_environment_variables().build()
        assert configuration["TEST"] == random_string

    def test_can_create_sections_from_environment_variables(self):
        random_string: str = str(uuid4())
        os.environ["ConnectionStrings:Database"] = random_string
        configuration: IConfiguration = ConfigurationBuilder().add_environment_variables().build()
        configurationSection: IConfigurationSection = configuration.get_section("ConnectionStrings")
        assert configurationSection is not None
        assert configurationSection["Database"] == random_string
        assert configuration.get_connection_string("Database") == random_string

    def test_non_existent_user_secrets_doesnt_crash(self):
        ConfigurationBuilder().add_user_secrets("asdf").build()

    def test_json_string(self):
        configuration: IConfiguration = ConfigurationBuilder().add_json_string(
            """
            {
                "simple": "asdf",
                "array": [
                    {
                        "Hey": 1,
                        "What": "Who?"
                    }
                ]
            }
            """
        ).build()

        assert configuration is not None
        assert configuration["simple"] == "asdf"
    
        assert configuration["array:0:Hey"] == "1"
        assert configuration["array:0:What"] == "Who?"
        
        assert configuration["nonexistent"] is None
        assert configuration["array:1"] is None
        
        array_section = configuration.get_section("array")
        assert array_section is not None
        assert array_section["0:Hey"] == "1"
        assert array_section["0:What"] == "Who?"
        
        array_children = list(array_section.get_children())
        assert len(array_children) == 1
        assert array_children[0]["Hey"] == "1"
        assert array_children[0]["What"] == "Who?"
        
        assert configuration["array:0:Hey"] == configuration.get_section("array").get_section("0")["Hey"]

    def test_json_file(self):
        current_dir = os.path.dirname(os.path.abspath(__file__))
        
        config_path = os.path.join(current_dir, "config.json")
        
        assert os.path.exists(config_path), f"Config file not found at {config_path}"

        configuration: IConfiguration = ConfigurationBuilder().add_json_file(config_path).build()
        assert configuration is not None
        assert configuration["simple"] == "asdf"
    
        assert configuration["array:0:Hey"] == "1"
        assert configuration["array:0:What"] == "Who?"
        
        assert configuration["nonexistent"] is None
        assert configuration["array:1"] is None
        
        array_section = configuration.get_section("array")
        assert array_section is not None
        assert array_section["0:Hey"] == "1"
        assert array_section["0:What"] == "Who?"
        
        array_children = list(array_section.get_children())
        assert len(array_children) == 1
        assert array_children[0]["Hey"] == "1"
        assert array_children[0]["What"] == "Who?"
        
        assert configuration["array:0:Hey"] == configuration.get_section("array").get_section("0")["Hey"]

    def test_json_file_not_optional(self):
        current_dir = os.path.dirname(os.path.abspath(__file__))
        config_path = os.path.join(current_dir, "config.json")
        
        configuration = ConfigurationBuilder().add_json_file(config_path, optional=False).build()
        assert configuration is not None
        assert configuration["simple"] == "asdf"

        with pytest.raises(FileNotFoundError):
            ConfigurationBuilder().add_json_file("non_existent.json", optional=False).build()

    def test_json_file_optional(self):
        configuration = ConfigurationBuilder().add_json_file("non_existent.json", optional=True).build()
        assert configuration is not None
        assert len(list(configuration.get_children())) == 0


# Licensed to the .NET Foundation under one or more agreements.
# The .NET Foundation licenses this file to you under the MIT license.

from __future__ import annotations

import os
from pathlib import Path
from typing import Optional

import userpaths

from . import ConfigurationProvider
from .abstractions import IConfigurationBuilder, IConfigurationProvider, IConfigurationSource
from .json import JsonConfigurationParser

class PathHelper:
    SECRETS_FILE_NAME: str = "secrets.json"

    @staticmethod
    def get_secrets_path_from_secrets_id(user_secrets_id: str) -> str:
        return PathHelper.internal_get_secrets_path_from_secrets_id(user_secrets_id, True)
    
    @staticmethod
    def internal_get_secrets_path_from_secrets_id(user_secrets_id: str, throw_if_no_root: bool):
        if not user_secrets_id:
            raise ValueError("user_secrets_id must be provided.")
        
        USER_SECRETS_FALLBACK_DIR: str = "DOTNET_USER_SECRETS_FALLBACK_DIR"

        app_data: Optional[str] = os.environ.get("APPDATA")
        root: Optional[str] = app_data
        root = root if root else os.environ.get("HOME")
        root = root if root else userpaths.get_appdata()
        root = root if root else userpaths.get_profile()
        root = root if root else os.environ.get(USER_SECRETS_FALLBACK_DIR)

        if not root:
            if throw_if_no_root:
                raise ValueError("Cannot determine valid user secrets location.")
            
            return ""
        
        return os.path.join(root, "Microsoft", "UserSecrets", user_secrets_id, PathHelper.SECRETS_FILE_NAME) if app_data else os.path.join(root, ".microsoft", "usersecrets", user_secrets_id, PathHelper.SECRETS_FILE_NAME)

class UserSecretsConfigurationProvider(ConfigurationProvider):
    def __init__(self, user_secrets_id: str):
        self.user_secrets_id = user_secrets_id

    def load(self) -> None:
        secrets_path = PathHelper.get_secrets_path_from_secrets_id(self.user_secrets_id)
        path: Path = Path(secrets_path)
        self.data = JsonConfigurationParser.parse_file(path)

class UserSecretsConfigurationSource(IConfigurationSource):
    def __init__(self, user_secrets_id: str):
        self.user_secrets_id = user_secrets_id

    def build(self, builder: IConfigurationBuilder) -> IConfigurationProvider:
        return UserSecretsConfigurationProvider(self.user_secrets_id)

def _add_user_secrets(self, user_secrets_id: str) -> IConfigurationBuilder:
    self.add(UserSecretsConfigurationSource(user_secrets_id))
    return self

IConfigurationBuilder.add_user_secrets = _add_user_secrets


# Licensed to the .NET Foundation under one or more agreements.
# The .NET Foundation licenses this file to you under the MIT license.

from __future__ import annotations

import os
from typing import Optional

from . import ConfigurationProvider
from .abstractions import ConfigurationPath, IConfigurationBuilder, IConfigurationProvider, IConfigurationSource

class EnvironmentVariablesConfigurationProvider(ConfigurationProvider):
    MY_SQL_SERVER_PREFIX: str = "MYSQLCONNSTR_"
    SQL_AZURE_SERVER_PREFIX: str = "SQLAZURECONNSTR_"
    SQL_SERVER_PREFIX: str = "SQLCONNSTR_"
    CUSTOM_CONNECTION_STRING_PREFIX: str = "CUSTOMCONNSTR_"

    _prefix: str
    _normalized_prefix: str

    def __init__(self, prefix: Optional[str] = None):
        """Initializes a new instance."""
        super().__init__()
        self._prefix = ""
        self._normalized_prefix = ""

        if prefix:
            self._prefix = prefix
            self._normalized_prefix = EnvironmentVariablesConfigurationProvider.normalize(prefix)

    def load(self) -> None:
        data: dict[str, Optional[str]] = dict()

        key: str
        value: Optional[str]
        for key, value in os.environ.items():
            transformed_key = key.casefold()
            if transformed_key.startswith(EnvironmentVariablesConfigurationProvider.MY_SQL_SERVER_PREFIX.casefold()):
                self.handle_matched_connection_string_prefix(data, EnvironmentVariablesConfigurationProvider.MY_SQL_SERVER_PREFIX, "MySql.Data.MySqlClient", key, value)
            elif transformed_key.startswith(EnvironmentVariablesConfigurationProvider.SQL_AZURE_SERVER_PREFIX.casefold()):
                self.handle_matched_connection_string_prefix(data, EnvironmentVariablesConfigurationProvider.SQL_AZURE_SERVER_PREFIX, "System.Data.SqlClient", key, value)
            elif transformed_key.startswith(EnvironmentVariablesConfigurationProvider.SQL_SERVER_PREFIX.casefold()):
                self.handle_matched_connection_string_prefix(data, EnvironmentVariablesConfigurationProvider.SQL_SERVER_PREFIX, "System.Data.SqlClient", key, value)
            elif transformed_key.startswith(EnvironmentVariablesConfigurationProvider.CUSTOM_CONNECTION_STRING_PREFIX.casefold()):
                self.handle_matched_connection_string_prefix(data, EnvironmentVariablesConfigurationProvider.CUSTOM_CONNECTION_STRING_PREFIX, None, key, value)
            else:
                self.add_if_normalized_key_matches_prefix(data, EnvironmentVariablesConfigurationProvider.normalize(key), value)

        self.data = data

    def handle_matched_connection_string_prefix(self, data: dict[str, Optional[str]], connection_string_prefix: str, provider: Optional[str], full_key: str, value: Optional[str]) -> None:
        normalized_key_without_connection_string_prefix: str = self.normalize(full_key[len(connection_string_prefix):])

        self.add_if_normalized_key_matches_prefix(data, f"ConnectionStrings:{normalized_key_without_connection_string_prefix}", value)
        if provider is not None:
            self.add_if_normalized_key_matches_prefix(data, f"ConnectionStrings:{normalized_key_without_connection_string_prefix}_ProviderName", provider)

    def add_if_normalized_key_matches_prefix(self, data: dict[str, Optional[str]], normalized_key: str, value: Optional[str]) -> None:
        if normalized_key.casefold().startswith(self._normalized_prefix.casefold()):
            data[normalized_key[len(self._normalized_prefix):]] = value

    @staticmethod
    def normalize(key: str) -> str:
        return key.replace("__", ConfigurationPath.KEY_DELIMITER)
    
class EnvironmentVariablesConfigurationSource(IConfigurationSource):
    """Represents environment variables as an IConfigurationSource."""
    prefix: Optional[str]

    def __init__(self):
        self.prefix = None

    def build(self, builder: IConfigurationBuilder) -> IConfigurationProvider:
        """Builds the EnvironmentVariablesConfigurationProvider for this source."""
        return EnvironmentVariablesConfigurationProvider(self.prefix)

def _add_environment_variables(self, prefix: Optional[str] = None) -> IConfigurationBuilder:
    """Adds an IConfigurationProvider that reads configuration values from environment variables."""
    if prefix:
        self.add(EnvironmentVariablesConfigurationSource(prefix))
    else:
        self.add(EnvironmentVariablesConfigurationSource())

    return self

IConfigurationBuilder.add_environment_variables = _add_environment_variables